#ifndef __LIVOX_MID_40_LIDAR__H
#define __LIVOX_MID_40_LIDAR__H
#include "Laser.h"
#include "livox_def.h"
#include "livox_sdk.h"
#include <map>

//大疆livox雷达，从Laser_base继承。
class CLivoxLaser :public Laser_base
{
public:
	//雷达设备状态，livox管理底层sdk，后台线程的工作状态
	typedef enum
	{
		K_DEVICE_STATE_DISCONNECT 		= 0,		//雷达设备状态	断开连接
		K_DEVICE_STATE_CONNECT 			= 1,		//雷达设备状态	连接正常
		K_DEVICE_STATE_SAMPLING 		= 2,		//雷达设备状态	正在扫描
		K_DEVICE_STATE_FAULT			= 3,		//雷达设备状态	故障
	} DeviceState;

	//雷达设备信息，
	typedef struct
	{
		uint8_t handle;					//雷达控制句柄
		DeviceState device_state;		//雷达设备状态
		DeviceInfo info;				//雷达基本信息
	} DeviceItem;


public:
	CLivoxLaser() = delete;
	CLivoxLaser(const CLivoxLaser& other) = delete;
	//唯一的构造函数，按照设备名称和雷达参数来创建实例。
	//input：id： 雷达设备的id，（唯一索引）
	//input：laser_param：雷达的参数，
	//注：利用protobuf创建 laser_parameter 类，然后从文件读取参数
	CLivoxLaser(int id, Laser_proto::laser_parameter laser_param);
	~CLivoxLaser();

	//雷达链接设备，为3个线程添加线程执行函数。
	virtual Error_manager connect_laser();
	//雷达断开链接，释放3个线程
	virtual Error_manager disconnect_laser();
	//对外的接口函数，负责接受并处理任务单，
	//input：p_laser_task 雷达任务单，基类的指针，指向子类的实例，（多态）
	//注：这个函数为虚函数，实际的处理任务的代码由子类重载并实现。
	virtual Error_manager execute_task(Task_Base* p_laser_task);
	//检查雷达状态,是否正常运行
	virtual Error_manager check_status();
	//雷达的启动接口函数， 让雷达进行扫描，一般需要子类重载，不同的雷达开始方式不同。
	virtual Error_manager start_scan();
	//雷达的停止接口函数， 让雷达停止扫描，一般需要子类重载，不同的雷达结束方式不同。
	virtual Error_manager stop_scan();
	//结束任务单，stop之后，要检查线程状态和数据结果，然后才能 end_task
	virtual Error_manager end_task();

	//判断雷达状态是否为待机，如果已经准备好，则可以执行任务。
	//子类重载 is_ready()，里面增加livox sdk后台线程状态的判断。
	virtual bool is_ready();

	//判断是否扫描完成
	virtual bool is_scan_complete();

	//往数据缓存里面添加雷达数据,
	virtual Error_manager push_livox_data(Binary_buf* p_binary_buf);

	//雷达帧数计数增加
	virtual Error_manager add_livox_frame(int add_count);


public://set and get
	void set_livox_device(uint8_t handle, DeviceState device_state = K_DEVICE_STATE_DISCONNECT, DeviceInfo info = DeviceInfo() );

	void set_handle(uint8_t handle);

	void set_device_state(DeviceState device_state);
	CLivoxLaser::DeviceState get_device_state();

	void set_device_info(DeviceInfo info);


protected:
	//接受二进制消息的功能函数，并将其存入 m_queue_laser_data, 每次只转化一个CBinaryData，
	// 纯虚函数，必须由子类重载，
	virtual Error_manager receive_buf_to_queue();
	//将二进制消息转化为三维点云的功能函数，并将其存入 任务单 , 每次只转化一个CBinaryData，
	// 纯虚函数，必须由子类重载，
	virtual Error_manager transform_buf_to_points();



protected:
	DeviceItem								m_livox_device;//雷达设备信息,handle在这里面.
	unsigned int							m_frame_count;//扫描当前帧数
	unsigned int							m_frame_maxnum;//扫描最大帧数
	Thread_safe_queue<Binary_buf*>			m_queue_livox_data;//雷达数据
	//注意了:雷达每帧数据都包含很多个三维点.
	//例如 livox雷达,每帧数据对应一个Binary_buf.对应100个三维点.
	//1000帧就是十万个三维点.

};

#endif

